@@ -1,7 +1,6 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2011 Edgewall Software, 2013-2020 the Babel team
+# Copyright (C) 2007-2011 Edgewall Software, 2013-2022 the Babel team
 # All rights reserved.
 #
 # This software is licensed as described in the file LICENSE, which
@@ -15,13 +14,12 @@
 import collections
 from optparse import OptionParser
 import os
+import pickle
 import re
 import sys
+import logging
 
-try:
-    from xml.etree import cElementTree as ElementTree
-except ImportError:
-    from xml.etree import ElementTree
+from xml.etree import ElementTree
 
 # Make sure we're using Babel source, and not some previously installed version
 CHECKOUT_ROOT = os.path.abspath(os.path.join(
@@ -32,7 +30,6 @@ BABEL_PACKAGE_ROOT = os.path.join(CHECKOUT_ROOT, "babel")
 sys.path.insert(0, CHECKOUT_ROOT)
 
 from babel import dates, numbers
-from babel._compat import pickle, text_type
 from babel.dates import split_interval_pattern
 from babel.localedata import Alias
 from babel.plural import PluralRule
@@ -62,16 +59,7 @@ NAME_MAP = {
     'timeFormats': 'time_formats'
 }
 
-
-def log(message, *args):
-    if args:
-        message = message % args
-    sys.stderr.write(message + '\r\n')
-    sys.stderr.flush()
-
-
-def error(message, *args):
-    log('ERROR: %s' % message, *args)
+log = logging.getLogger("import_cldr")
 
 
 def need_conversion(dst_filename, data_dict, source_filename):
@@ -125,7 +113,7 @@ def _extract_plural_rules(file_path):
     for elem in prsup.findall('.//plurals/pluralRules'):
         rules = []
         for rule in elem.findall('pluralRule'):
-            rules.append((rule.attrib['count'], text_type(rule.text)))
+            rules.append((rule.attrib['count'], str(rule.text)))
         pr = PluralRule(rules)
         for locale in elem.attrib['locales'].split():
             rule_dict[locale] = pr
@@ -182,10 +170,19 @@ def main():
         '-j', '--json', dest='dump_json', action='store_true', default=False,
         help='also export debugging JSON dumps of locale data'
     )
+    parser.add_option(
+        '-q', '--quiet', dest='quiet', action='store_true', default=bool(os.environ.get('BABEL_CLDR_QUIET')),
+        help='quiesce info/warning messages',
+    )
 
     options, args = parser.parse_args()
     if len(args) != 1:
         parser.error('incorrect number of arguments')
+
+    logging.basicConfig(
+        level=(logging.ERROR if options.quiet else logging.INFO),
+    )
+
     return process_data(
         srcdir=args[0],
         destdir=BABEL_PACKAGE_ROOT,
@@ -236,13 +233,13 @@ def parse_global(srcdir, sup):
     for map_zone in sup_windows_zones.findall('.//windowsZones/mapTimezones/mapZone'):
         if map_zone.attrib.get('territory') == '001':
             win_mapping[map_zone.attrib['other']] = map_zone.attrib['type'].split()[0]
-        for tzid in text_type(map_zone.attrib['type']).split():
-            _zone_territory_map[tzid] = text_type(map_zone.attrib['territory'])
+        for tzid in str(map_zone.attrib['type']).split():
+            _zone_territory_map[tzid] = str(map_zone.attrib['territory'])
     for key_elem in bcp47_timezone.findall('.//keyword/key'):
         if key_elem.attrib['name'] == 'tz':
             for elem in key_elem.findall('type'):
                 if 'deprecated' not in elem.attrib:
-                    aliases = text_type(elem.attrib['alias']).split()
+                    aliases = str(elem.attrib['alias']).split()
                     tzid = aliases.pop(0)
                     territory = _zone_territory_map.get(tzid, '001')
                     territory_zones.setdefault(territory, []).append(tzid)
@@ -300,8 +297,8 @@ def parse_global(srcdir, sup):
             all_currencies[cur_code].add(region_code)
         region_currencies.sort(key=_currency_sort_key)
         territory_currencies[region_code] = region_currencies
-    global_data['all_currencies'] = dict([
-        (currency, tuple(sorted(regions))) for currency, regions in all_currencies.items()])
+    global_data['all_currencies'] = {
+        currency: tuple(sorted(regions)) for currency, regions in all_currencies.items()}
 
     # Explicit parent locales
     for paternity in sup.findall('.//parentLocales/parentLocale'):
@@ -342,7 +339,7 @@ def _process_local_datas(sup, srcdir, destdir, force=False, dump_json=False):
     region_items = sorted(regions.items())
     for group, territory_list in region_items:
         for territory in territory_list:
-            containers = territory_containment.setdefault(territory, set([]))
+            containers = territory_containment.setdefault(territory, set())
             if group in territory_containment:
                 containers |= territory_containment[group]
             containers.add(group)
@@ -383,8 +380,10 @@ def _process_local_datas(sup, srcdir, destdir, force=False, dump_json=False):
             territory = '001'  # world
         regions = territory_containment.get(territory, [])
 
-        log('Processing %s (Language = %s; Territory = %s)',
-            filename, language, territory)
+        log.info(
+            'Processing %s (Language = %s; Territory = %s)',
+            filename, language, territory,
+        )
 
         locale_id = '_'.join(filter(None, [
             language,
@@ -392,6 +391,7 @@ def _process_local_datas(sup, srcdir, destdir, force=False, dump_json=False):
         ]))
 
         data['locale_id'] = locale_id
+        data['unsupported_number_systems'] = set()
 
         if locale_id in plural_rules:
             data['plural_form'] = plural_rules[locale_id]
@@ -432,6 +432,13 @@ def _process_local_datas(sup, srcdir, destdir, force=False, dump_json=False):
         parse_character_order(data, tree)
         parse_measurement_systems(data, tree)
 
+        unsupported_number_systems_string = ', '.join(sorted(data.pop('unsupported_number_systems')))
+        if unsupported_number_systems_string:
+            log.warning('%s: unsupported number systems were ignored: %s' % (
+                locale_id,
+                unsupported_number_systems_string,
+            ))
+
         write_datafile(data_filename, data, dump_json=dump_json)
 
 
@@ -440,21 +447,14 @@ def _should_skip_number_elem(data, elem):
     Figure out whether the numbering-containing element `elem` is in a currently
     non-supported (i.e. currently non-Latin) numbering system.
 
-    If it is, a warning is raised.
-
-    :param data: The root data element, for formatting the warning.
+    :param data: The root data element, for stashing the warning.
     :param elem: Element with `numberSystem` key
     :return: Boolean
     """
     number_system = elem.get('numberSystem', 'latn')
 
     if number_system != 'latn':
-        log('%s: Unsupported number system "%s" in <%s numberSystem="%s">' % (
-            data['locale_id'],
-            number_system,
-            elem.tag,
-            number_system,
-        ))
+        data['unsupported_number_systems'].add(number_system)
         return True
 
     return False
@@ -548,22 +548,22 @@ def parse_dates(data, tree, sup, regions, territory):
     zone_formats = data.setdefault('zone_formats', {})
     for elem in tree.findall('.//timeZoneNames/gmtFormat'):
         if not _should_skip_elem(elem):
-            zone_formats['gmt'] = text_type(elem.text).replace('{0}', '%s')
+            zone_formats['gmt'] = str(elem.text).replace('{0}', '%s')
             break
     for elem in tree.findall('.//timeZoneNames/regionFormat'):
         if not _should_skip_elem(elem):
-            zone_formats['region'] = text_type(elem.text).replace('{0}', '%s')
+            zone_formats['region'] = str(elem.text).replace('{0}', '%s')
             break
     for elem in tree.findall('.//timeZoneNames/fallbackFormat'):
         if not _should_skip_elem(elem):
             zone_formats['fallback'] = (
-                text_type(elem.text).replace('{0}', '%(0)s').replace('{1}', '%(1)s')
+                str(elem.text).replace('{0}', '%(0)s').replace('{1}', '%(1)s')
             )
             break
     for elem in tree.findall('.//timeZoneNames/fallbackRegionFormat'):
         if not _should_skip_elem(elem):
             zone_formats['fallback_region'] = (
-                text_type(elem.text).replace('{0}', '%(0)s').replace('{1}', '%(1)s')
+                str(elem.text).replace('{0}', '%(0)s').replace('{1}', '%(1)s')
             )
             break
     time_zones = data.setdefault('time_zones', {})
@@ -571,22 +571,22 @@ def parse_dates(data, tree, sup, regions, territory):
         info = {}
         city = elem.findtext('exemplarCity')
         if city:
-            info['city'] = text_type(city)
+            info['city'] = str(city)
         for child in elem.findall('long/*'):
-            info.setdefault('long', {})[child.tag] = text_type(child.text)
+            info.setdefault('long', {})[child.tag] = str(child.text)
         for child in elem.findall('short/*'):
-            info.setdefault('short', {})[child.tag] = text_type(child.text)
+            info.setdefault('short', {})[child.tag] = str(child.text)
         time_zones[elem.attrib['type']] = info
     meta_zones = data.setdefault('meta_zones', {})
     for elem in tree.findall('.//timeZoneNames/metazone'):
         info = {}
         city = elem.findtext('exemplarCity')
         if city:
-            info['city'] = text_type(city)
+            info['city'] = str(city)
         for child in elem.findall('long/*'):
-            info.setdefault('long', {})[child.tag] = text_type(child.text)
+            info.setdefault('long', {})[child.tag] = str(child.text)
         for child in elem.findall('short/*'):
-            info.setdefault('short', {})[child.tag] = text_type(child.text)
+            info.setdefault('short', {})[child.tag] = str(child.text)
         meta_zones[elem.attrib['type']] = info
 
 
@@ -598,7 +598,7 @@ def parse_calendar_months(data, calendar):
         for width in ctxt.findall('monthWidth'):
             width_type = width.attrib['type']
             widths = ctxts.setdefault(width_type, {})
-            for elem in width.getiterator():
+            for elem in width:
                 if elem.tag == 'month':
                     _import_type_text(widths, elem, int(elem.attrib['type']))
                 elif elem.tag == 'alias':
@@ -616,7 +616,7 @@ def parse_calendar_days(data, calendar):
         for width in ctxt.findall('dayWidth'):
             width_type = width.attrib['type']
             widths = ctxts.setdefault(width_type, {})
-            for elem in width.getiterator():
+            for elem in width:
                 if elem.tag == 'day':
                     _import_type_text(widths, elem, weekdays[elem.attrib['type']])
                 elif elem.tag == 'alias':
@@ -634,7 +634,7 @@ def parse_calendar_quarters(data, calendar):
         for width in ctxt.findall('quarterWidth'):
             width_type = width.attrib['type']
             widths = ctxts.setdefault(width_type, {})
-            for elem in width.getiterator():
+            for elem in width:
                 if elem.tag == 'quarter':
                     _import_type_text(widths, elem, int(elem.attrib['type']))
                 elif elem.tag == 'alias':
@@ -649,7 +649,7 @@ def parse_calendar_eras(data, calendar):
     for width in calendar.findall('eras/*'):
         width_type = NAME_MAP[width.tag]
         widths = eras.setdefault(width_type, {})
-        for elem in width.getiterator():
+        for elem in width:
             if elem.tag == 'era':
                 _import_type_text(widths, elem, type=int(elem.attrib.get('type')))
             elif elem.tag == 'alias':
@@ -670,23 +670,23 @@ def parse_calendar_periods(data, calendar):
             for day_period in day_period_width.findall('dayPeriod'):
                 period_type = day_period.attrib['type']
                 if 'alt' not in day_period.attrib:
-                    dest_dict[period_type] = text_type(day_period.text)
+                    dest_dict[period_type] = str(day_period.text)
 
 
 def parse_calendar_date_formats(data, calendar):
     date_formats = data.setdefault('date_formats', {})
     for format in calendar.findall('dateFormats'):
-        for elem in format.getiterator():
+        for elem in format:
             if elem.tag == 'dateFormatLength':
                 type = elem.attrib.get('type')
                 if _should_skip_elem(elem, type, date_formats):
                     continue
                 try:
                     date_formats[type] = dates.parse_pattern(
-                        text_type(elem.findtext('dateFormat/pattern'))
+                        str(elem.findtext('dateFormat/pattern'))
                     )
                 except ValueError as e:
-                    error(e)
+                    log.error(e)
             elif elem.tag == 'alias':
                 date_formats = Alias(_translate_alias(
                     ['date_formats'], elem.attrib['path'])
@@ -696,17 +696,17 @@ def parse_calendar_date_formats(data, calendar):
 def parse_calendar_time_formats(data, calendar):
     time_formats = data.setdefault('time_formats', {})
     for format in calendar.findall('timeFormats'):
-        for elem in format.getiterator():
+        for elem in format:
             if elem.tag == 'timeFormatLength':
                 type = elem.attrib.get('type')
                 if _should_skip_elem(elem, type, time_formats):
                     continue
                 try:
                     time_formats[type] = dates.parse_pattern(
-                        text_type(elem.findtext('timeFormat/pattern'))
+                        str(elem.findtext('timeFormat/pattern'))
                     )
                 except ValueError as e:
-                    error(e)
+                    log.error(e)
             elif elem.tag == 'alias':
                 time_formats = Alias(_translate_alias(
                     ['time_formats'], elem.attrib['path'])
@@ -717,15 +717,15 @@ def parse_calendar_datetime_skeletons(data, calendar):
     datetime_formats = data.setdefault('datetime_formats', {})
     datetime_skeletons = data.setdefault('datetime_skeletons', {})
     for format in calendar.findall('dateTimeFormats'):
-        for elem in format.getiterator():
+        for elem in format:
             if elem.tag == 'dateTimeFormatLength':
                 type = elem.attrib.get('type')
                 if _should_skip_elem(elem, type, datetime_formats):
                     continue
                 try:
-                    datetime_formats[type] = text_type(elem.findtext('dateTimeFormat/pattern'))
+                    datetime_formats[type] = str(elem.findtext('dateTimeFormat/pattern'))
                 except ValueError as e:
-                    error(e)
+                    log.error(e)
             elif elem.tag == 'alias':
                 datetime_formats = Alias(_translate_alias(
                     ['datetime_formats'], elem.attrib['path'])
@@ -733,7 +733,7 @@ def parse_calendar_datetime_skeletons(data, calendar):
             elif elem.tag == 'availableFormats':
                 for datetime_skeleton in elem.findall('dateFormatItem'):
                     datetime_skeletons[datetime_skeleton.attrib['id']] = (
-                        dates.parse_pattern(text_type(datetime_skeleton.text))
+                        dates.parse_pattern(str(datetime_skeleton.text))
                     )
 
 
@@ -746,7 +746,7 @@ def parse_number_symbols(data, tree):
         for elem in symbol_elem.findall('./*'):
             if _should_skip_elem(elem):
                 continue
-            number_symbols[elem.tag] = text_type(elem.text)
+            number_symbols[elem.tag] = str(elem.text)
 
 
 def parse_decimal_formats(data, tree):
@@ -763,15 +763,13 @@ def parse_decimal_formats(data, tree):
                 continue
             for pattern_el in elem.findall('./decimalFormat/pattern'):
                 pattern_type = pattern_el.attrib.get('type')
-                pattern = numbers.parse_pattern(text_type(pattern_el.text))
+                pattern = numbers.parse_pattern(str(pattern_el.text))
                 if pattern_type:
                     # This is a compact decimal format, see:
                     # https://www.unicode.org/reports/tr35/tr35-45/tr35-numbers.html#Compact_Number_Formats
 
                     # These are mapped into a `compact_decimal_formats` dictionary
                     # with the format {length: {count: {multiplier: pattern}}}.
-
-                    # TODO: Add support for formatting them.
                     compact_decimal_formats = data.setdefault('compact_decimal_formats', {})
                     length_map = compact_decimal_formats.setdefault(length_type, {})
                     length_count_map = length_map.setdefault(pattern_el.attrib['count'], {})
@@ -790,7 +788,7 @@ def parse_scientific_formats(data, tree):
             type = elem.attrib.get('type')
             if _should_skip_elem(elem, type, scientific_formats):
                 continue
-            pattern = text_type(elem.findtext('scientificFormat/pattern'))
+            pattern = str(elem.findtext('scientificFormat/pattern'))
             scientific_formats[type] = numbers.parse_pattern(pattern)
 
 
@@ -804,7 +802,7 @@ def parse_percent_formats(data, tree):
             type = elem.attrib.get('type')
             if _should_skip_elem(elem, type, percent_formats):
                 continue
-            pattern = text_type(elem.findtext('percentFormat/pattern'))
+            pattern = str(elem.findtext('percentFormat/pattern'))
             percent_formats[type] = numbers.parse_pattern(pattern)
 
 
@@ -819,15 +817,15 @@ def parse_currency_names(data, tree):
                 continue
             if 'count' in name.attrib:
                 currency_names_plural.setdefault(code, {})[
-                    name.attrib['count']] = text_type(name.text)
+                    name.attrib['count']] = str(name.text)
             else:
-                currency_names[code] = text_type(name.text)
+                currency_names[code] = str(name.text)
         for symbol in elem.findall('symbol'):
             if 'draft' in symbol.attrib or 'choice' in symbol.attrib:  # Skip drafts and choice-patterns
                 continue
             if symbol.attrib.get('alt'):  # Skip alternate forms
                 continue
-            currency_symbols[code] = text_type(symbol.text)
+            currency_symbols[code] = str(symbol.text)
 
 
 def parse_unit_patterns(data, tree):
@@ -841,6 +839,9 @@ def parse_unit_patterns(data, tree):
             unit_type = unit.attrib['type']
             unit_and_length_patterns = unit_patterns.setdefault(unit_type, {}).setdefault(unit_length_type, {})
             for pattern in unit.findall('unitPattern'):
+                if pattern.attrib.get('case', 'nominative') != 'nominative':
+                    # Skip non-nominative cases.
+                    continue
                 unit_and_length_patterns[pattern.attrib['count']] = _text(pattern)
 
             per_unit_pat = unit.find('perUnitPattern')
@@ -853,9 +854,26 @@ def parse_unit_patterns(data, tree):
 
         for unit in elem.findall('compoundUnit'):
             unit_type = unit.attrib['type']
-            compound_patterns.setdefault(unit_type, {})[unit_length_type] = (
-                _text(unit.find('compoundUnitPattern'))
-            )
+            compound_unit_info = {}
+            compound_variations = {}
+            for child in unit:
+                if child.attrib.get('case', 'nominative') != 'nominative':
+                    # Skip non-nominative cases.
+                    continue
+                if child.tag == "unitPrefixPattern":
+                    compound_unit_info['prefix'] = _text(child)
+                elif child.tag == "compoundUnitPattern":
+                    compound_variations[None] = _text(child)
+                elif child.tag == "compoundUnitPattern1":
+                    compound_variations[child.attrib.get('count')] = _text(child)
+            if compound_variations:
+                compound_variation_values = set(compound_variations.values())
+                if len(compound_variation_values) == 1:
+                    # shortcut: if all compound variations are the same, only store one
+                    compound_unit_info['compound'] = next(iter(compound_variation_values))
+                else:
+                    compound_unit_info['compound_variations'] = compound_variations
+            compound_patterns.setdefault(unit_type, {})[unit_length_type] = compound_unit_info
 
 
 def parse_date_fields(data, tree):
@@ -867,7 +885,7 @@ def parse_date_fields(data, tree):
             rel_time_type = rel_time.attrib['type']
             for pattern in rel_time.findall('relativeTimePattern'):
                 type_dict = date_fields[field_type].setdefault(rel_time_type, {})
-                type_dict[pattern.attrib['count']] = text_type(pattern.text)
+                type_dict[pattern.attrib['count']] = str(pattern.text)
 
 
 def parse_interval_formats(data, tree):
@@ -880,7 +898,7 @@ def parse_interval_formats(data, tree):
             interval_formats[None] = elem.text
         elif elem.tag == "intervalFormatItem":
             skel_data = interval_formats.setdefault(elem.attrib["id"], {})
-            for item_sub in elem.getchildren():
+            for item_sub in elem:
                 if item_sub.tag == "greatestDifference":
                     skel_data[item_sub.attrib["id"]] = split_interval_pattern(item_sub.text)
                 else:
@@ -903,14 +921,14 @@ def parse_currency_formats(data, tree):
                     type = '%s:%s' % (type, curr_length_type)
                 if _should_skip_elem(elem, type, currency_formats):
                     continue
-                for child in elem.getiterator():
+                for child in elem.iter():
                     if child.tag == 'alias':
                         currency_formats[type] = Alias(
                             _translate_alias(['currency_formats', elem.attrib['type']],
                                              child.attrib['path'])
                         )
                     elif child.tag == 'pattern':
-                        pattern = text_type(child.text)
+                        pattern = str(child.text)
                         currency_formats[type] = numbers.parse_pattern(pattern)
 
 
@@ -921,7 +939,7 @@ def parse_currency_unit_patterns(data, tree):
             continue
         for unit_pattern_elem in currency_formats_elem.findall('./unitPattern'):
             count = unit_pattern_elem.attrib['count']
-            pattern = text_type(unit_pattern_elem.text)
+            pattern = str(unit_pattern_elem.text)
             currency_unit_patterns[count] = pattern
 
 
@@ -940,10 +958,10 @@ def parse_day_period_rules(tree):
                 type = rule.attrib["type"]
                 if type in ("am", "pm"):  # These fixed periods are handled separately by `get_period_id`
                     continue
-                rule = _compact_dict(dict(
-                    (key, _time_to_seconds_past_midnight(rule.attrib.get(key)))
+                rule = _compact_dict({
+                    key: _time_to_seconds_past_midnight(rule.attrib.get(key))
                     for key in ("after", "at", "before", "from", "to")
-                ))
+                })
                 for locale in locales:
                     dest_list = day_periods.setdefault(locale, {}).setdefault(ruleset_type, {}).setdefault(type, [])
                     dest_list.append(rule)
